namespace RoboidControl {

    /// <summary>
    /// Message notifiying that a Thing no longer exists
    /// </summary>
    public class DestroyMsg : IMessage {
        /// <summary>
        /// The message ID
        /// </summary>
        public const byte Id = 0x20;
        /// <summary>
        /// The length of the message
        /// </summary>
        public const byte length = 3;
        /// <summary>
        ///  The network ID of the thing
        /// </summary>
        public byte networkId;
        /// <summary>
        /// The ID of the thing
        /// </summary>
        public byte thingId;

        /// <summary>
        /// Create a message for sending
        /// </summary>
        /// <param name="networkId">The network ID of the thing</param>
        /// <param name="thingId">The ID of the thing</param>
        public DestroyMsg(byte networkId, Thing thing) {
            this.networkId = networkId;
            this.thingId = thing.id;
        }
        /// @copydoc Passer::RoboidControl::IMessage::IMessage(byte[] buffer)
        public DestroyMsg(byte[] buffer) : base(buffer) {
            this.networkId = buffer[1];
            this.thingId = buffer[2];
         }

        /// @copydoc Passer::RoboidControl::IMessage::Serialize
        public override byte Serialize(ref byte[] buffer) {
            if (buffer.Length < DestroyMsg.length)
                return 0;
            
            byte ix = 0;
            buffer[ix++] = DestroyMsg.Id;
            buffer[ix++] = this.networkId;
            buffer[ix++] = this.thingId;

            return ix;
        }

    }

}