namespace RoboidControl {

    /// <summary>
    /// Message for communicating the URL for a model of the thing
    /// </summary>
    public class ModelUrlMsg : IMessage {
        /// <summary>
        /// The message ID
        /// </summary>
        public const byte Id = 0x90; // (144) Model URL
        /// <summary>
        /// The length of the message without th URL itself
        /// </summary>
        public const byte length = 4;
        /// <summary>
        /// The network ID of the thing
        /// </summary>
        public byte networkId;
        /// <summary>
        /// The ID of the thing
        /// </summary>
        public byte thingId;
        /// <summary>
        /// The length of the url string, excluding the null terminator
        /// </summary>
        public byte urlLength;
        /// <summary>
        /// The URL of the model
        /// </summary>
        public string url = null;

        /// <summary>
        /// Create a new message for sending
        /// </summary>
        /// <param name="networkId">The network ID of the thing</param>
        /// <param name="thing">The thing for which to send the model URL</param>
        public ModelUrlMsg(byte networkId, Thing thing) {
            this.networkId = networkId;
            this.thingId = thing.id;
            this.urlLength = (byte)thing.modelUrl.Length;
            this.url = thing.modelUrl;
        }
        /// <summary>
        /// Create a new message for sending
        /// </summary>
        /// <param name="networkId">The network ID of the thing</param>
        /// <param name="thingId">The ID of the thing</param>
        /// <param name="url">The URL to send</param>
        // public ModelUrlMsg(byte networkId, byte thingId, string url) {
        //     this.networkId = networkId;
        //     this.thingId = thingId;
        //     this.urlLength = (byte)url.Length;
        //     this.url = url;
        // }
        /// @copydoc Passer::RoboidControl::IMessage::IMessage(byte[] buffer)
        public ModelUrlMsg(byte[] buffer) {
            byte ix = 1;
            this.networkId = buffer[ix++];
            this.thingId = buffer[ix++];

            this.urlLength = buffer[ix++];
            this.url = System.Text.Encoding.UTF8.GetString(buffer, (int)ix, this.urlLength);
        }

        /// @copydoc Passer::RoboidControl::IMessage::Serialize
        public override byte Serialize(ref byte[] buffer) {
            if (string.IsNullOrEmpty(this.url))
                return 0;

#if DEBUG
            System.Console.WriteLine($"Send ModelUrlMsg [{this.networkId}/{this.thingId}] {this.urlLength} {this.url}");
#endif
            byte ix = 0;
            buffer[ix++] = ModelUrlMsg.Id;
            buffer[ix++] = this.networkId;
            buffer[ix++] = this.thingId; // Thing Id

            buffer[ix++] = (byte)this.url.Length;
            for (int urlIx = 0; urlIx < this.url.Length; urlIx++)
                buffer[ix++] = (byte)url[urlIx];
            return ix;
        }

    }

}