namespace RoboidControl {

    /// <summary>
    /// Message for communicating the name of a thing
    /// </summary>
    public class NameMsg : IMessage {
        /// <summary>
        /// The message ID
        /// </summary>
        public const byte Id = 0x91; // 145
        /// <summary>
        /// The length of the message without the name string
        /// </summary>
        public const byte length = 4;
        /// <summary>
        /// The network ID of the thing
        /// </summary>
        public byte networkId;
        /// <summary>
        /// The ID of the thing
        /// </summary>
        public byte thingId;

        /// <summary>
        /// The length of the name, excluding null terminator
        /// </summary>
        public byte nameLength;
        /// <summary>
        /// The name of the thing, not terminated with a null character
        /// </summary>
        public string name = "";

        /// <summary>
        /// Create a new message for sending
        /// </summary>
        /// <param name="networkId">The network ID of the thing</param>
        /// <param name="thing">The thing</param>
        public NameMsg(byte networkId, Thing thing) {
            this.networkId = networkId;
            this.thingId = thing.id;
            this.name = thing.name;
            this.nameLength = (byte)this.name.Length;
        }
        /// <summary>
        /// Create a new message for sending
        /// </summary>
        /// <param name="networkId">The network ID of the thing</param>
        /// <param name="thingId">The ID of the thing</param>
        /// <param name="name">The name of the thing</param>
        // public NameMsg(byte networkId, byte thingId, string name) {
        //     this.networkId = networkId;
        //     this.thingId = thingId;
        //     this.name = name;
        // }
        /// @copydoc Passer::RoboidControl::IMessage::IMessage(byte[] buffer)
        public NameMsg(byte[] buffer) {
            byte ix = 1;
            this.networkId = buffer[ix++];
            this.thingId = buffer[ix++];
            this.nameLength = buffer[ix++];
            this.name = System.Text.Encoding.UTF8.GetString(buffer, (int)ix, this.nameLength);
        }

        /// @copydoc Passer::RoboidControl::IMessage::Serialize
        public override byte Serialize(ref byte[] buffer) {
            if (buffer.Length < NameMsg.length + this.name.Length || string.IsNullOrEmpty(this.name))
                return 0;

#if DEBUG
            System.Console.WriteLine($"Send NameMsg [{this.networkId}/{this.thingId}] {this.nameLength} {this.name}");
#endif
            byte ix = 0;
            buffer[ix++] = NameMsg.Id;
            buffer[ix++] = this.networkId;
            buffer[ix++] = this.thingId;
            buffer[ix++] = this.nameLength;
            for (int nameIx = 0; nameIx < this.nameLength; nameIx++)
                buffer[ix++] = (byte)this.name[nameIx];

            return ix;
        }
    }

}