namespace RoboidControl {

    /// <summary>
    /// A message communicating the network ID for that participant
    /// </summary>
    public class NetworkIdMsg : IMessage {
        /// <summary>
        /// The message ID
        /// </summary>
        public const byte Id = 0xA1;
        /// <summary>
        /// The length of the message
        /// </summary>
        public const byte length = 2;
        /// <summary>
        /// The network ID for the participant
        /// </summary>
        public byte networkId;

        /// <summary>
        /// Create a new message for sending
        /// </summary>
        /// <param name="networkId">The network ID for the participant</param>
        public NetworkIdMsg(byte networkId) {
            this.networkId = networkId;
        }
        /// @copydoc Passer::RoboidControl::IMessage::IMessage(byte[] buffer)
        public NetworkIdMsg(byte[] buffer) {
            this.networkId = buffer[1];
        }

        /// @copydoc Passer::RoboidControl::IMessage::Serialize
        public override byte Serialize(ref byte[] buffer) {
            if (buffer.Length < NetworkIdMsg.length)
                return 0;

#if DEBUG
            System.Console.WriteLine($"Send NetworkIdMsg {this.networkId}");
#endif
            buffer[0] = NetworkIdMsg.Id;
            buffer[1] = this.networkId;
            return NetworkIdMsg.length;
        }
    }

}