namespace RoboidControl {

    /// <summary>
    /// Message providing generic information about a Thing
    /// </summary>
    public class ThingMsg : IMessage {
        /// <summary>
        /// The message ID
        /// </summary>
        public const byte id = 0x80;
        /// <summary>
        /// The length of the message
        /// </summary>
        public const byte length = 5;
        /// <summary>
        /// The network ID of the thing
        /// </summary>
        public byte networkId;
        /// <summary>
        /// The ID of the thing
        /// </summary>
        public byte thingId;
        /// <summary>
        /// The Thing.Type of the thing
        /// </summary>
        public byte thingType;
        /// <summary>
        /// The parent of the thing in the hierarachy. This is null for root Things
        /// </summary>
        public byte parentId;

        /// <summary>
        /// Create a message for sending
        /// </summary>
        /// <param name="networkId">The network ID of the thing</param>
        /// <param name="thing">The thing</param>
        public ThingMsg(byte networkId, Thing thing) {
            this.networkId = networkId;
            this.thingId = thing.id;
            this.thingType = thing.type;
            if (thing.parent != null)
                this.parentId = thing.parent.id;
            else
                this.parentId = 0;
        }
        /// <summary>
        /// Create a message for sending
        /// </summary>
        /// <param name="networkId">The network ID of the thing</param>
        /// <param name="thingId">The ID of the thing</param>
        /// <param name="thingType">The type of thing</param>
        /// <param name="parentId">The parent of the thing</param>
        public ThingMsg(byte networkId, byte thingId, byte thingType, byte parentId) {
            this.networkId = networkId;
            this.thingId = thingId;
            this.thingType = thingType;
            this.parentId = parentId;
        }
        /// <summary>
        /// Create a message for receiving
        /// </summary>
        /// <param name="buffer">The byte array to parse</param>
        public ThingMsg(byte[] buffer) {
            uint ix = 1;
            this.networkId = buffer[ix++];
            this.thingId = buffer[ix++];
            this.thingType = buffer[ix++];
            this.parentId = buffer[ix];
        }

        /// <summary>
        /// Serialize the message into a byte array
        /// </summary>
        /// <param name="buffer">The buffer to serialize into</param>
        /// <returns>The length of the message in the bufer. 0 when the buffer was too small</returns>
        public override byte Serialize(ref byte[] buffer) {
            if (buffer.Length < ThingMsg.length)
                return 0;
                
#if DEBUG
            System.Console.WriteLine($"Send ThingMsg [{this.networkId}/{this.thingId}] {this.thingType} {this.parentId}");
#endif
            byte ix = 0;
            buffer[ix++] = ThingMsg.id;
            buffer[ix++] = this.networkId;
            buffer[ix++] = this.thingId;
            buffer[ix++] = this.thingType;
            buffer[ix++] = this.parentId;
            return ThingMsg.length;
        }
    }

}